/* tcp-web-client.cc
// TE816 - Modelagem e Avaliação de Desempenho
// Pós Graduação em Engenharia Elétrica
// Universidade Federal do Paraná
// This souce was developed as basis for homework 
//
// -- 10/11/2018
// ==================================
// Prof. Carlos Marcelo Pedroso
// pedroso@eletrica.ufpr.br
// http://www.eletrica.ufpr.br/pedroso
// Universidade Federal do Paraná
// Departamento de Engenharia Elétrica
// Centro Politécnico
// CEP: 81531-990 - Curitiba - PR - Brasil
 */


#ifndef TCP_WEB_CLIENT
#define TCP_WEB_CLIENT


#include "ns3/application.h"
#include "ns3/event-id.h"
#include "ns3/ptr.h"
#include "ns3/ipv4-address.h"
#include "ns3/ipv6-address.h"
#include "ns3/traced-callback.h"

namespace ns3 {

class Socket;
class Packet;

/**
 * \ingroup tcpweb
 * \brief A Tcp web client
 *
 * Every packet sent should be returned by the server and received here.
 */
class TcpWebClient : public Application
{
public:
  static TypeId GetTypeId (void);

  TcpWebClient ();

  virtual ~TcpWebClient ();

  /**
   * \param ip destination address
   * \param port destination port
   */
  void SetRemote (Address ip, uint16_t port);

  void SetDataSize (uint32_t dataSize);
  uint32_t GetDataSize (void) const;
  void SetFill (std::string fill);
  void SetFill (uint8_t fill, uint32_t dataSize);
  void SetFill (uint8_t *fill, uint32_t fillSize, uint32_t dataSize);

  // mainly to the automated test
  uint32_t GetPacketsSent (void) { return m_sent; };
  uint64_t GetBytesSent (void) { return m_bytesSent; };
  uint32_t GetPacketsReceivedBack (void) { return m_recvBack; };
  uint64_t GetBytesReceivedBack (void) { return m_bytesRecvBack; };

protected:
  virtual void DoDispose (void);

private:

  virtual void StartApplication (void);
  virtual void StopApplication (void);

  void ScheduleTransmit (Time dt);
  void Send (void);

  void ReceivePacket (Ptr<Socket> socket);

  uint32_t m_count;
  Time m_interval;
  uint32_t m_size;

  uint32_t m_dataSize;
  uint8_t *m_data;

  uint32_t m_sent;
  uint64_t m_bytesSent;
  uint32_t m_recvBack;
  uint64_t m_bytesRecvBack;
  Ptr<Socket> m_socket;
  Address m_peerAddress;
  uint16_t m_peerPort;
  EventId m_sendEvent;
  /// Callbacks for tracing the packet Tx events
  TracedCallback<Ptr<const Packet> > m_txTrace;
};

} // namespace ns3


#endif /* TCP_WEB_CLIENT */
